@@ -16,9 +16,9 @@ inline
 wxIcon getRtsIcon() //see FFS/app_icon.h
 {
     assert(loadImage("RealTimeSync").GetWidth () == loadImage("RealTimeSync").GetHeight() &&
-           loadImage("RealTimeSync").GetWidth() == fastFromDIP(128));
+           loadImage("RealTimeSync").GetWidth() == dipToScreen(128));
     wxIcon icon;
-    icon.CopyFromBitmap(loadImage("RealTimeSync", fastFromDIP(64)));
+    icon.CopyFromBitmap(loadImage("RealTimeSync", dipToScreen(64)));
     return icon;
 
 }
