@@ -165,8 +165,6 @@ MainDialog::MainDialog(const Zstring& cfgFilePath) :
 
     onSystemShutdownRegister(onBeforeSystemShutdownCookie_);
 
-    Center(); //needs to be re-applied after a dialog size change! (see addFolder() within setConfiguration())
-
     if (startWatchingImmediately) //start watch mode directly
     {
         wxCommandEvent dummy2(wxEVT_COMMAND_BUTTON_CLICKED);
@@ -175,6 +173,13 @@ MainDialog::MainDialog(const Zstring& cfgFilePath) :
     }
     else
     {
+        //GetSizer()->SetSizeHints(this); //~=Fit() + SetMinSize() => already called by setConfiguration() -> insertAddFolder()
+#ifdef __WXGTK3__
+        Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+        //Hide(); -> avoids old position flash before Center() on GNOME but causes hang on KDE? https://freefilesync.org/forum/viewtopic.php?t=10103#p42404
+#endif
+        Center(); //apply *after* dialog size change!
+
         Show();
         m_buttonStart->SetFocus(); //don't "steal" focus if program is running from sys-tray"
     }
@@ -257,7 +262,15 @@ void MainDialog::onStart(wxCommandEvent& event)
             break;
     }
 
+    //need to center in case of "startWatchingImmediately"
+#ifdef __WXGTK3__
+    Show(); //GTK3 size calculation requires visible window: https://github.com/wxWidgets/wxWidgets/issues/16088
+    //Hide(); -> avoids old position flash before Center() on GNOME but causes hang on KDE? https://freefilesync.org/forum/viewtopic.php?t=10103#p42404
+#endif
+    Center(); //apply *after* dialog size change!
+
     Show(); //don't show for CancelReason::requestExit
+
     Raise();
     m_buttonStart->SetFocus();
 }
