@@ -7,15 +7,13 @@
 #include "graph.h"
 #include <cassert>
 #include <algorithm>
-//#include <numeric>
 #include <zen/basic_math.h>
 #include <zen/scope_guard.h>
-//#include <zen/perf.h>
 
 using namespace zen;
 
 
-//todo: support zoom via mouse wheel?
+//TODO: support zoom via mouse wheel?
 
 namespace zen
 {
@@ -47,7 +45,6 @@ wxColor getDefaultColor(size_t pos)
 {
     switch (pos % 10)
     {
-        //*INDENT-OFF*
         case 0: return {  0,  69, 134}; //blue
         case 1: return {255,  66,  14}; //red
         case 2: return {255, 211,  32}; //yellow
@@ -58,7 +55,6 @@ wxColor getDefaultColor(size_t pos)
         case 7: return {174, 207,   0}; //light green
         case 8: return { 75,  31, 111}; //purple
         case 9: return {255, 149,  14}; //orange
-        //*INDENT-ON*
     }
     assert(false);
     return *wxBLACK;
@@ -139,7 +135,8 @@ int widenRange(double& valMin, double& valMax, //in/out
 }
 
 
-void drawXLabel(wxDC& dc, double xMin, double xMax, int blockCount, const ConvertCoord& cvrtX, const wxRect& graphArea, const wxRect& labelArea, const LabelFormatter& labelFmt)
+void drawXLabel(wxDC& dc, double xMin, double xMax, int blockCount, const ConvertCoord& cvrtX, const wxRect& graphArea,
+                const wxRect& labelArea, const LabelFormatter& labelFmt, const wxColor& colGridLine)
 {
     assert(graphArea.width == labelArea.width && graphArea.x == labelArea.x);
     if (blockCount <= 0)
@@ -153,7 +150,7 @@ void drawXLabel(wxDC& dc, double xMin, double xMax, int blockCount, const Conver
         const int x = graphArea.x + cvrtX.realToScreenRound(valX);
 
         //draw grey vertical lines
-        clearArea(dc, {x - dipToWxsize(1) / 2, graphArea.y, dipToWxsize(1), graphArea.height}, wxColor(192, 192, 192)); //light grey => not accessible! but no big deal...
+        clearArea(dc, {x - dipToWxsize(1) / 2, graphArea.y, dipToWxsize(1), graphArea.height}, colGridLine);
 
         //draw x axis labels
         const wxString label = labelFmt.formatText(valX, valRangePerBlock);
@@ -163,7 +160,8 @@ void drawXLabel(wxDC& dc, double xMin, double xMax, int blockCount, const Conver
 }
 
 
-void drawYLabel(wxDC& dc, double yMin, double yMax, int blockCount, const ConvertCoord& cvrtY, const wxRect& graphArea, const wxRect& labelArea, const LabelFormatter& labelFmt)
+void drawYLabel(wxDC& dc, double yMin, double yMax, int blockCount, const ConvertCoord& cvrtY, const wxRect& graphArea,
+                const wxRect& labelArea, const LabelFormatter& labelFmt, const wxColor& colGridLine)
 {
     assert(graphArea.height == labelArea.height && graphArea.y == labelArea.y);
     if (blockCount <= 0)
@@ -177,7 +175,7 @@ void drawYLabel(wxDC& dc, double yMin, double yMax, int blockCount, const Conver
         const double valY = yMin + i * valRangePerBlock; //step over raw data, not graph area pixels, to not lose precision
         const int y = graphArea.y + cvrtY.realToScreenRound(valY);
 
-        clearArea(dc, {graphArea.x, y - dipToWxsize(1) / 2, graphArea.width, dipToWxsize(1)}, wxColor(192, 192, 192)); //light grey => not accessible! but no big deal...
+        clearArea(dc, {graphArea.x, y - dipToWxsize(1) / 2, graphArea.width, dipToWxsize(1)}, colGridLine);
 
         //draw y axis labels
         const wxString label = labelFmt.formatText(valY, valRangePerBlock);
@@ -436,13 +434,13 @@ Graph2D::Graph2D(wxWindow* parent,
                  long style,
                  const wxString& name) : wxPanel(parent, winid, pos, size, style, name)
 {
+    //https://wiki.wxwidgets.org/Flicker-Free_Drawing
+    SetBackgroundStyle(wxBG_STYLE_PAINT); //get's rid of needless wxEVT_ERASE_BACKGROUND
     Bind(wxEVT_PAINT, [this](wxPaintEvent& event) { onPaintEvent(event); });
-    Bind(wxEVT_SIZE,  [this](wxSizeEvent& event) { Refresh(); event.Skip(); });
-    Bind(wxEVT_ERASE_BACKGROUND, [](wxEraseEvent& event) {}); //https://wiki.wxwidgets.org/Flicker-Free_Drawing
+    Bind(wxEVT_SIZE,  [this](wxSizeEvent&  event) { Refresh(); event.Skip(); });
 
-    //SetDoubleBuffered(true); slow as hell!
-
-    SetBackgroundStyle(wxBG_STYLE_PAINT);
+    //perf: WS_EX_COMPOSITED vs BufferedPaintDC doesn't seem to matter. Even for 200 FPS graph, CPU consumption is barely noticeable!
+    //MSWDisableComposited(); -> see comment in grid.cpp
 
     Bind(wxEVT_LEFT_DOWN,          [this](wxMouseEvent& event) { onMouseLeftDown(event); });
     Bind(wxEVT_MOTION,             [this](wxMouseEvent& event) { onMouseMovement(event); });
@@ -453,8 +451,7 @@ Graph2D::Graph2D(wxWindow* parent,
 
 void Graph2D::onPaintEvent(wxPaintEvent& event)
 {
-    //wxAutoBufferedPaintDC dc(this); -> this one happily fucks up for RTL layout by not drawing the first column (x = 0)!
-    BufferedPaintDC dc(*this, doubleBuffer_);
+    DynBufPaintDC dc(*this, doubleBuffer_);
     render(dc);
 }
 
@@ -569,7 +566,7 @@ void Graph2D::render(wxDC& dc) const
     assert(attr_.yLabelpos == YLabelPos::none || attr_.labelFmtY);
 
     //paint graph background (excluding label area)
-    drawFilledRectangle(dc, graphArea, attr_.colorBack, getBorderColor(), dipToWxsize(1));
+    drawFilledRectangle(dc, graphArea, attr_.colorBack, attr_.colorGridLine, dipToWxsize(1));
     graphArea.Deflate(dipToWxsize(1));
 
     //set label areas respecting graph area border!
@@ -759,8 +756,8 @@ void Graph2D::render(wxDC& dc) const
             }
 
             //3. draw labels and background grid
-            if (attr_.labelFmtX) drawXLabel(dc, minX, maxX, blockCountX, cvrtX, graphArea, xLabelArea, *attr_.labelFmtX);
-            if (attr_.labelFmtY) drawYLabel(dc, minY, maxY, blockCountY, cvrtY, graphArea, yLabelArea, *attr_.labelFmtY);
+            if (attr_.labelFmtX) drawXLabel(dc, minX, maxX, blockCountX, cvrtX, graphArea, xLabelArea, *attr_.labelFmtX, attr_.colorGridLine);
+            if (attr_.labelFmtY) drawYLabel(dc, minY, maxY, blockCountY, cvrtY, graphArea, yLabelArea, *attr_.labelFmtY, attr_.colorGridLine);
 
             //4. finally draw curves
             {
