@@ -5,7 +5,7 @@
 // *****************************************************************************
 
 #include "zstring.h"
-    #include <glib.h>
+    //#include <glib.h>
     #include "sys_error.h"
 
 using namespace zen;
@@ -104,7 +104,7 @@ Zstring getUpperCaseNonAscii(const Zstring& str)
 
         UtfDecoder<char> decoder(strNorm.c_str(), strNorm.size());
         while (const std::optional<impl::CodePoint> cp = decoder.getNext())
-            codePointToUtf<char>(::g_unichar_toupper(*cp), [&](char c) { output += c; }); //don't use std::towupper: *incomplete* and locale-dependent!
+            codePointToUtf<char>(::g_unichar_toupper(*cp), [&](const char c) { output += c; }); //don't use std::towupper: *incomplete* and locale-dependent!
 
         static_assert(sizeof(impl::CodePoint) == sizeof(gunichar));
         return output;
@@ -176,7 +176,7 @@ std::weak_ordering compareNatural(const Zstring& lhs, const Zstring& rhs)
 {
     try
     {
-        /* Unicode normal forms:
+        /* Unicode Normalization Forms:
               Windows: CompareString() ignores NFD/NFC differences and converts to NFD
               Linux:  g_unichar_toupper() can't ignore differences
               macOS:  CFStringCompare() considers differences */
@@ -242,7 +242,7 @@ std::weak_ordering compareNatural(const Zstring& lhs, const Zstring& rhs)
                 continue;
             }
 
-            //compare full junks of text: consider unicode encoding!
+            //compare full junks of text: consider Unicode encoding!
             const char* textBeginL = strL++;
             const char* textBeginR = strR++; //current char is neither white space nor digit at this point!
             while (strL != strEndL && !isWhiteSpace(*strL) && !isDigit(*strL)) ++strL;
